################################################################################
# Licensed Materials - Property of IBM Copyright IBM Corporation 2012. All Rights Reserved.
# U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP
# Schedule Contract with IBM Corp.
################################################################################
require "java"

begin
  java_import org.eclipse.core.runtime.NullProgressMonitor
  java_import com.ibm.team.build.internal.PasswordHelper
  java_import com.ibm.team.build.internal.scm.RepositoryManager
	
  java_import com.ibm.team.repository.client.ITeamRepository
  java_import com.ibm.team.repository.client.ILicenseClient
  java_import com.ibm.team.repository.client.TeamPlatform
  
rescue Exception => e
   Dir["#{File.dirname(__FILE__)}/libs/**/*.jar"].each { |jar| require jar }
   java_import org.eclipse.core.runtime.NullProgressMonitor
   java_import com.ibm.team.build.internal.PasswordHelper
   java_import com.ibm.team.build.internal.scm.RepositoryManager

   java_import com.ibm.team.repository.client.ITeamRepository
   java_import com.ibm.team.repository.client.ILicenseClient
   java_import com.ibm.team.repository.client.TeamPlatform
end

module DevopsCore
  module License

    def License.is_authorized?(repository_url, user_id, password_hash={})
      credential_args = {
        :password => ''
      }.merge(password_hash)

      if (repository_url.nil? || repository_url.empty? || user_id.nil? || user_id.empty?) 
        raise ArgumentError, "A URL or User ID to a Jazz Server were not found. Please include both of these arguments to continue."
      end
      
      if !credential_args[:passwordFile].nil? and File.exists?(credential_args[:passwordFile])
        # Decrypt password file if it exists
        pw_file = java.io.File.new(credential_args[:passwordFile])
        pw = PasswordHelper.getPassword(pw_file)
        credential_args[:password] = pw
      end

      repo = License.getTeamRepository(
        repository_url, 
        user_id, 
        credential_args[:password]
      )

      licenseClient = repo.getClientLibrary(
        com.ibm.team.repository.client.ILicenseClient.java_class
      );

      licenseClient.assertLicense("com.ibm.devops.delivery.createEnvironment", nil);  
      
    end

    def License.getTeamRepository(repository_url, user_id, password)
      TeamPlatform.startup()

      repo_manager = RepositoryManager.createForUsernameAndPassword(
        repository_url,
        user_id,
        password,
        nil
      )

      team_repository = repo_manager.getRepository(repository_url, true)

      repo_manager.login(team_repository, NullProgressMonitor.new);
      
      return team_repository
    end
  end
end

# possibly decrypt pw
# perform license check