require 'weaver/core/logging'
require 'erubis'
require 'weaver/utils/indent'

module Weaver
  module Core
    #
    # Defines a module that can be mixed in to all the core model class if code generation is 
    # required. The to_weaver method expect to find an ERB template of the core model class in the
    # directory data/templates name after the full class name. For example a
    # class: Weaver::Core::OperatingSystem is expected to have a template file 
    # weaver_core_operatingsystem.erb in the templates directory. The basic rule is that the template
    # name is lowercase and the :: are replaced by '_'. 
    # 
    # Author: Florian Rosenberg
    # Author: Ed Snible
    #
    module Generation 
      include Weaver::Core  
      
      # Return a String containing a Weaver description of the entity Generation is mixed with
      def to_weaver        
        # derive template name according to convention
        template_file = "data/templates/#{self.class.name.gsub("::", "_").downcase}.erb"
        file = File.join(File.dirname(__FILE__), "..", "..", "..", template_file)
        unless File.exists?(file)
          raise WeaverError, "Cannot find Erubis template file #{template_file} for class #{self.class.name}."
        end
        gen = Erubis::Eruby.new(File.read(file)).result(binding)
        return gen + "\n" unless gen.index("\n") # Single line results are just the result plus a newline 
        return Weaver::Utils::Indent.fixup(gen)
      end

    end
  end
end