require 'weaver/core/base.rb'

module Weaver
  module Core
    # Represents user credentials (typically an OS or LDAP username/password and metadata)
    # @see UserStatement
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    class User < Base     
      include Weaver::Core::Generation 
      
      # * +id+ A symbol for the Node, e.g. +:node1+
      # * +parent+ The enclosing Base
      def initialize(id, parent)
        super(id, parent)
        @type = :user
        amd[:username] = Attribute.new(:username, self, true)
        amd[:username].value = nil
        amd[:password] = Attribute.new(:password, self, true)
        amd[:password].type = :password
        amd[:password].value = nil
      end
      
      def validate_collect(out_messages, desired_phases = nil)
        @@logger.debug "Validating user '#{id}'"
		    @@logger.debug "  has username '#{username}'"
#		    @@logger.debug "  has password '#{password}'"		    
		    super
		    
        if password.nil?
          @@logger.debug "     NIL PASSWORD, see #{@weaver_origin}"
        else
          if password.respond_to?(:length) && password.length == 0
            @@logger.debug "     EMPTY PASSWORD, set at #{@password.instance_variable_get(:@weaver_origin)}"
          end
        end
      end
      
    end
  end
end