module Weaver
  module Dsl
    # Mix in ArtifactContainer in statements that may contain {ArtifactStatement}
    #
    # Author:: Fabio Oliveira
    module ArtifactContainer
      include Weaver::Core
      
      # * *Args* :
      #   -+base+ the AbstractStatement being mixed with
      def self.included(base)
        base.register_statement_handler([:artifact_handler])
      end

      # * *Args* :
      #   -+sym_or_model+ <Symbol representing Weaver type, e.g. <code>:artifact</code> | Core::Artifact
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def artifact_handler(sym_or_model, id, &block)
        return nil unless is_a?(Weaver::Dsl::ArtifactContainer) && (sym_or_model.kind_of?(Artifact) || root_statement.model.weaver_extended_type?(sym_or_model, Artifact))
        return nil unless id.kind_of?(Symbol) 
        
        prototype = sym_or_model.kind_of?(Symbol) ? root_statement.model.prototype_instance(sym_or_model, id, model, weaver_stackframe) : sym_or_model 
        artifact(id, prototype, &block)
      end

      # If prototype is nil, we will create a core Artifact, otherwise inherit attributes,
      # default attribute values, and validators from the prototype
      def artifact(id, prototype = nil, &block)
        @@logger.debug "artifact[(#{prototype.nil? ? 'artifact' : prototype.type}): #{id}]"
        
        return if fail_id_redefinition(id, Artifact)
        
        artifact_model = @model.artifacts[id]
        if artifact_model.nil?
          artifact_model = prototype || root_statement.model.prototype_instance(:artifact, id, model, weaver_stackframe)
          @model.artifacts[id] = artifact_model
        else
          root_statement.model.widen(artifact_model, prototype) unless prototype.nil?
        end
        
        artifact_statement = ArtifactStatement.new(id, self)
        artifact_statement.model.weaver_origin = weaver_stackframe
        @model.artifacts[id] = artifact_statement.model
        
        @@logger.debug "Processing artifact with id '#{id}'"
        artifact_statement.instance_eval(&block)

        if artifact_statement.model.source_path.nil?
          model.add_validation_failure("Artifact :#{id} has no source path")
          return
        end
        if artifact_statement.model.target_path.nil?
          model.add_validation_failure("Artifact :#{id} has no target path")
          return
        end
        
        return artifact_model
      end
    end
  end
end