require 'weaver/dsl/abstract_statement'
require 'weaver/core/artifact'

module Weaver
  module Dsl
    # The ArtifactStatement class implements the processing of the language element 'artifact'.
    #
    # Author:: Fabio Oliveira
    class ArtifactStatement < AbstractStatement
      # The model represents the Core::Artifact that corresponds to this statement.
      #attr_accessor :model
      
      # * +id+ Symbol representing the Core::Artifact
      # * +parent_statement+ The enclosing AbstractStatement
      def initialize(id, parent_statement)
       # @@logger.debug "artifact[id: #{id}]"
       # @model = Artifact.new(id)
       # @parent = parent_statement
       @@logger.debug "artifact[id: #{id} on #{parent_statement.model.type} :#{parent_statement.model.id}]"
       super(id, :artifact, parent_statement)
      end
      
      # Parse the 'source' of an artifact
      # * +location+ - It is either a \Symbol indicating whether the artifact resides locally (:local) or in the Weaver service (:service),
      # or a \String representing the source path to the artifact.
      # * +path+ - \Hash containing the source path to the artifact. The key used is :path. 
      # Examples: 
      #  source :local, :path => 'rules/artifacts/my_script.sh'
      #  source 'rules/artifacts/my_script.sh'  
      # The last example defaults to :service
      def source(location, path = nil)
        if location.kind_of?(String)
          # Defaults to :service. Location contains the path as a String
          @@logger.debug "Using default location: :service"
          @model.location = :service
          raise WeaverError.new("Artifact :#{@model.id} has an invalid format for 'source'") unless (location.kind_of?(String) && !location.nil?)
          @@logger.debug "source path: #{location}"
          @model.source_path = location
        else
          # We expect a Symbol and a Hash
          raise WeaverError.new("Artifact :#{@model.id} has an invalid location. Valid values: :local or :service") unless (location == :local || location == :service)
          @@logger.debug "location: #{location}"
          @model.location = location
          raise WeaverError.new("Artifact :#{@model.id} has an invalid format for 'source'") unless path.kind_of?(Hash)
          @@logger.debug "path hash: #{path.inspect}"
          @@logger.debug "source path: #{path[:path]}"
          @model.source_path = path[:path]
        end
      end

      # Parse the 'target' of an artifact.
      # * +path+ - \String representing the target path to the artifact 
      # Example: target '/opt/weaver/rules/artifacts/my_script.sh'
      def target(path)
        raise WeaverError.new("Artifact :#{@model.id} has an invalid format for 'target'") unless path.kind_of?(String)
        @@logger.debug "target path: #{path}"
        @model.target_path = path
      end
      
      # Parse the 'exec' keyword of an artifact. Its value should be a Boolean indicating whether or not the artifact is executable.
      def exec(is_exec)
        raise WeaverError.new("Artifact :#{@model.id} has an invalid format for 'exec'. Valid values: true or false.") unless (is_exec == true || is_exec == false)
        @@logger.debug "exec: #{is_exec}"
        @model.exec = is_exec
      end
    end
  end
end