require 'weaver/dsl/abstract_statement'
require 'weaver/dsl/group_container'

module Weaver
  module Dsl
    
    # Parses a {Weaver::Core::Group}
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    class GroupStatement < AbstractStatement
      include GroupContainer
    
      # * +id+ Symbol representing the entity, or a Core::Group which is the entity
      # * +parent_statement+ The enclosing AbstractStatement
      def initialize(id, parent_statement)
        @@logger.debug "group[id: #{id}]"
        super(id, :group, parent_statement)
      end
    
      # Parse +members+ language statement
      def members(*references)
        references.each do |reference|
          entity = resolve_reference(reference)
          @model.members[entity.id] = entity
          #@model.members[node] = nil 
        end
        # Later during validation we will look up the symbols we saved here
      end
      
      # reference can be a Symbol (e.g. :jke_web)
      # reference can be a Statement referring to a Weaver entity (e.g. app_server.c1)
      def resolve_reference(ref)
        return ref.model if ref.kind_of?(AbstractStatement)
        
        return model.add_validation_failure("Group member #{ref} must be a symbol or entity") unless ref.kind_of?(Symbol)
        
        retval = @model.scope.flatten_children.find { |entity| entity.id == ref }
        
        return model.add_validation_failure("Group member not found #{ref}") if retval.nil?
        return retval
      end
      
    end
    
  end

end
