module Weaver
  module Dsl
    
    # The PolicyContainer mixin.  Include this mixin in statements that
    # should support {PolicyStatement}s.
    #
    # Author:: Thomas Gschwind <thg@zurich.ibm.com>
    # Copyright:: IBM
    module PolicyContainer
      include Weaver::Core
      
      # * *Args* :
      #   -+base+ the AbstractStatement being mixed with
      def self.included(base)
        base.register_statement_handler([:policy_handler])
      end

      # * *Args* :
      #   -+sym_or_model+ <Symbol representing Weaver type, e.g. <code>:policy</code> | Core::Policy
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def policy_handler(sym_or_model, id, &block)
        return nil unless is_a?(Weaver::Dsl::PolicyContainer) 
        return nil unless sym_or_model.kind_of?(Policy) || root_statement.model.weaver_extended_type?(sym_or_model, Policy)
        return nil unless id.kind_of?(Symbol)
         
        prototype = sym_or_model.kind_of?(Symbol) ? root_statement.model.prototype_instance(sym_or_model, id, model, weaver_stackframe) : sym_or_model 
        policy(id, prototype, &block)
      end
      
      # TODO: check with Ed whether this is fine
      def copy_attributes(target, source)
        target.amd={}
        source.amd.each { |id, attribute|
          dup = attribute.clone()
          dup.scope = target
          target.amd[id] = dup
        }
      end
      
      # If prototype is nil, we will create a core Policy, otherwise inherit attributes,
      # default attribute values, and validators from the prototype
      def policy(id, prototype = nil, &block)
        @@logger.debug "policy[(#{prototype.id.to_s+') ' unless prototype.nil?}:#{id}]"

        # Go up the model (via the parent) to
        # * find the node statement
        #   necessary for policies that need the node (e.g., its IP address)
        current = self
        node = nil

        while current != nil do
          if current.kind_of? NodeStatement
            node = current
          end
          unless current.respond_to? :parent
            break
          end
          current = current.parent
        end
        
        policy = PolicyStatement.new(id, self)
        unless prototype.nil?        
          policy.model = prototype
          policy.model.id = id
          policy.model.scope = @model
        end
        @model.policies[id] = policy.model
        policy.instance_eval &block unless block.nil?

        unless node.nil?
          # 1. for each foo_policy create a role foo_role
          policy_type = policy.model.type.to_s
          policy_type['_policy']=''
          role_name = (policy_type+'_role')
          policy_typeid_prefix= policy_type+'.'+policy.model.id.to_s
        
#          puts "policy_type=#{policy_type} {{{{"
#          puts "  role_name=#{role_name}, policy_typeid_prefix=#{policy_typeid_prefix}"
          
          # 2. add chef automation for that role
          node.chef_automation(role_name.to_sym) {
            def make_property(prefix, postfix, value)
              property(
                (prefix+"."+postfix).gsub(/[^_a-zA-Z0-9]/,'_').to_sym => value,
                :name => (prefix+"."+postfix)
              )
            end
            
            # compute the set of policies
            if model.role.nil?
              role role_name
              policies = [policy.model.id.to_s]
            else
              policies = model.policies + [policy.model.id.to_s]
            end
#            puts "  policies are #{policies}"
            property :policies => policies, :name => policy_type+'.policies'
          
            # WAIT properties:
            #   description, server, destination, pid_regex, sampling_interval, sampling_count
            # ITM properties:
            #   server (ex tems), description, condition, agent, sampling_rate, force, 
            #   severity - string [Fatal, Critical, Minor, Warning, Harmless, Informational, Unknown]
            policy.model.amd.each { |id, e|
              # Ignore name (internal attribute), username, and password.
              # The latter to are passed directly to the service activating
              # the policy using the - u and -p parameters
              if id!=:name && id!=:username && id!=:password
                value = e.value.nil? ? '' : e.value
                make_property(policy_typeid_prefix, id.to_s, value)
              end
            }
            
#            puts "}}}} policy_type=#{policy_type}"
          }
        end
        
        return policy.model
      end
    end
  end
end