require 'weaver/core/logging'
require 'weaver/core/provider'

module Weaver
  module Dsl
    # Mix in ProviderContainerStatement with statements that may contain {ProviderStatement}
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    module ProviderContainerStatement
      include Weaver::Core
      
      # Create and optionally parse Provider
      #   - +id_or_model+ ProviderStatement | Symbol naming new Provider
      #   - +block+ optional block of additional Provider metadata
      def provider(id_or_statement, &block)
        if id_or_statement.kind_of?(ProviderStatement)
          provider_model = id_or_statement.model
          model.provisioner = provider_model 
        else
          id = id_or_statement
          provider_model = model.providers[id]
          if provider_model.nil?
            provider_model = root_statement.model.provider_instance(id, model)
            model.providers[id] = provider_model
          end
          
          provider_model.weaver_origin = weaver_stackframe
          
          unless block.nil?
            begin
              provider_statement = ProviderStatement.new(provider_model, self)
              provider_statement.instance_eval &block
            end
          end
          
          # Is this the first provider?  If so, mix immediately
          if model.providers.size == 1
            provider_model.mixin_provider_attributes(model)
          end
        end
    
        return provider_model
      end
      
    end

  end

end