require 'weaver/core/logging'

module Weaver
  module Dsl
    # Mix in StorageContainerStatement with statements that may contain storage.  There is no model type
    # for storage, storage is represented by a {Base} defined in builtins.weaver with specific attributes.
    #
    # Author:: Ed Snible
    # Copyright:: IBM
    module StorageContainerStatement
      include Weaver::Core
      
      # * *Args* :
      #   -+sym_or_model+ <Symbol representing Weaver type, e.g. <code>:volume</code> | Core::Base
      #   -+id+ Symbol naming entity
      #   -+block+ optional block of additional metadata
      def storage_handler(sym_or_model, id, &block)
        return nil unless is_a?(Weaver::Dsl::StorageContainer) && (sym_or_model.weaver_instance_of?(:storage))
        return nil unless id.kind_of?(Symbol) 
        
        prototype = sym_or_model.kind_of?(Symbol) ? root_statement.model.prototype_instance(sym_or_model, id, model, weaver_stackframe) : sym_or_model 
        storage(id, prototype, &block)
      end
      
      # Create and optionally parse Base
      #   - +id_or_model+ Symbol naming new Base
      #   - +block+ optional block of additional Base metadata
      def storage(id, prototype = nil, &block)
        @@logger.debug "storage[(#{prototype.nil? ? 'storage' : prototype.type}): #{id}]"
         
         return if fail_id_redefinition(id, Base)
         
         storage_model = model.storage[id]
         if storage_model.nil?
           storage_model = prototype || root_statement.model.prototype_instance(:storage, id, model, weaver_stackframe)
           model.storage[id] = storage_model
         else
           root_statement.model.widen(storage_model, prototype) unless prototype.nil?
         end
         
         unless block.nil?
           storage = BaseStatement.new(storage_model, self)
           
           begin
             storage.instance_eval &block
           rescue ArgumentError => ae
             # For example the statement 'component' without an ID yields
             # 'wrong number of arguments (0 for 1)'
             storage_model.add_validation_failure(" #{ae.message}", weaver_stackframe(ae.backtrace))
             @@logger.warn "Caught #{ae.inspect} processing 'storage' statement, backtrace=#{ae.backtrace}"
           rescue NameError => ne
             # For example the statement 'N1.name' yields NameError uninitialized constant
             storage_model.add_validation_failure(" #{ne.message}", weaver_stackframe(ne.backtrace))
             @@logger.warn "Caught #{ne.inspect} processing 'storage' statement, backtrace=#{ne.backtrace}"
           end
         end
 
         return storage_model
      end
      
    end

  end

end