require 'weaver/core/logging'
require 'weaver/core/node'
require 'weaver/core/weaver_error'
require 'weaver/coordination/coordinated_transaction_manager'
require 'weaver/deployers/deployer'
require 'weaver/execution/actions/action'
require 'weaver/providers/delegating_provider'

module Weaver
  module Execution
    module Actions
    
      # Deletes a list of nodes.
      # An empty list of nodes means that all nodes should be deleted
      
      class DeleteNodesAction < Weaver::Execution::Actions::Action
        
        def initialize (nodes = [])
          @nodes = nodes
        end # end def initialize
              
        # TODO figure out how this works for WCC
        def execute  (properties, execute_parameters = {})
          raise Weaver::Core::WeaverError.new("Coordination transaction identifier required.") if execute_parameters[:coordination_id].nil?

          tx = plan.transaction
            
          failed_nodes = [];
          @nodes.each do |node|
            # set state of node to "deleting"
            node.amd[:config_status] = Weaver::Coordination::CONFIG_STATUS::DELETING
            tx.write_data("#{node.fully_qualified_id}.config_status", Weaver::Coordination::CONFIG_STATUS::DELETING)
            
            # get deployer
            provisioner = node.provisioner || Weaver::Providers::DelegatingProvider.new()
            provisioner = node.effective_provisioner(:cloud) || Weaver::Providers::DelegatingProvider.new()
            deployer = provisioner.deployer(properties, &@reporter)
            begin 
              deployer.delete_node(node, nil, properties)
              # TODO this should probably be done afterwards by a monitoring thread
              node.amd[:config_status] = Weaver::Coordination::CONFIG_STATUS::DELETED
              node.amd[:vm_status] = Weaver::Coordination::CONFIG_STATUS::DELETE_FAILED
            rescue Weaver::Core::WeaverError
              @@logger.error "Failed to delete node #{node.id}"
              failed_nodes << node.fully_qualified_id;
              node.amd[:config_status] = Weaver::Coordination::CONFIG_STATUS::DELETE_FAILED
              node.amd[:vm_status] = Weaver::Coordination::CONFIG_STATUS::DELETE_FAILED
            end
            
            # record status to coordinator (waiting processes can proceed)
            tx.write_data("#{node.fully_qualified_id}.config_status", node.amd[:config_status])
            tx.write_data("#{node.fully_qualified_id}.vm_status", node.amd[:vm_status])
          end # @nodes.each
          
          raise WeaverError.new("Delete failures for nodes: #{failed_nodes.inspect}") unless failed_nodes.empty?
          
        end # end def execute
        
        def to_s
          retval = "DeleteNodesAction["
          retval << "#{@nodes.map{|node| node.fully_qualified_id}.join(';')}"
          retval << "]"
          
          return retval
        end
        
      end # class DeleteNodesAction
    
    end # module Action
  end # end module Execution
end # mmodule Weaver