require 'rubygems'
require 'chef'
require 'chef/handler'
require 'weaver_coordination'

class Chef
  class Handler
    class NodeConfigStatus < Chef::Handler
      include Weaver::Coordination::Logging

      def initialize(coordinator_hosts = 'localhost', coordinator_port = '2181', coordination_id = "", node_id = "", fq_node_id = "")
        @coordinator_hosts = coordinator_hosts
        @coordinator_port = coordinator_port
        @coordination_id = coordination_id
        @node_id = node_id
        @fq_node_id = fq_node_id
        @@logger.info "NodeConfigStatus using coordinator host: #{@coordinator_hosts.inspect}"
        @@logger.info "NodeConfigStatus using coordinator port: #{@coordinator_port}"
        @@logger.info "NodeConfigStatus coordination transaction id: #{@coordination_id}"
        @@logger.info "NodeConfigStatus node id: #{@node_id}"
        @@logger.info "NodeConfigStatus fully qualified node id: #{@fq_node_id}"
        Weaver::Coordination::System.instance.set_property(:zookeeper_hosts, @coordinator_hosts)
        Weaver::Coordination::System.instance.set_property(:zookeeper_port, @coordinator_port)
        @coordinator_instance = Weaver::Coordination::CoordinatedTransactionManager.instance.join_transaction(@coordination_id)
      end

      def report
        # Call out to coordinator to set node status
        if success?
          @@logger.info "Successfully updated VM 'config_status' to 'configured'"
          @coordinator_instance.write_data("#{@fq_node_id}.config_status", Weaver::Coordination::CONFIG_STATUS::CONFIGURED)
        else
          @@logger.info "Successfully updated VM 'config_status' to 'failed'"
          @coordinator_instance.write_data("#{@fq_node_id}.config_status", Weaver::Coordination::CONFIG_STATUS::FAILED)
        end

        # +run_status+ is a value object with all of the run status data
        message = "#{run_status.formatted_exception}\n"
        # Join the backtrace lines. Coerce to an array just in case.
        message << Array(backtrace).join("\n")
      end
    end
  end
end